<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla06000IntegracaoReceitaController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName;
    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $connection;
    protected $dataSourceManager;

    public function __construct($crudService, $parameterBag, $entityManager, $environment, $newCode, $connection, $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->entityManager     = $entityManager;
        $this->environment       = $environment;
        $this->newCode           = $newCode;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
    }
	
	
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $this->entityManager->beginTransaction();
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                $this->entityManager->commit();
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        }
    }
    
    //apaga lista de valores recebendo string como parametro
    public function apagarListaValores($valores){
        try{
            if(substr($valores, 0,9) == "_NRLISTA="){
                $nrlista = substr($valores, 9);
                $this->deleteLista($nrlista);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function filtraPratos($request, $response){
        $filter = $request->getFilterCriteria();
        $conditions = $filter->getConditions();
        $codigos = array();
        // se o valor é concatenado, passa pela função 'criarListaValores'
        // para verificar se é necessário criar ou não lista
        foreach($conditions as $key => $current){
            if(!is_array($conditions[$key]["value"]) && strpos($conditions[$key]["value"], '_')){
                $conditions[$key]['value'] = $this->criarListaValores($conditions[$key]['value']);
                array_push($codigos, $conditions[$key]['value']);
            }
        }
        
        $dataSet = $this->crudService->getFilteredDataSet('pla06000_pratos', $conditions);
        $response->addDataSet($dataSet);
        
        //apaga as listas geradas para o filtro 
        if(sizeof($codigos) > 0){
            foreach($codigos as $code){
                $this->apagarListaValores($code);
            }
        }
        
    }
    
    public function exporta($request, $response){
        $row = $request->getRow();
        $lista = $this->criaListaPratos($row['PRATOS']);
        $nrLog = array();
        try{
            foreach($row['P_CDFILIDEST'] as $filial){
                $this->connection->beginTransaction();
                try{
                    $procedure = new StoredProcedure($this->connection, 'INTEGRA_RECEITA_CENTRALIZADA');
                    $procedure->addParam(new Param('P_CDFILIORIG',  Param::PARAM_INPUT,  $row['P_CDFILIORIG'], Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_CDFILIDEST',  Param::PARAM_INPUT,  $filial,              Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_NRLISTA',     Param::PARAM_INPUT,  $lista,               Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_IDTIPORECE',  Param::PARAM_INPUT,  $row['P_IDTIPORECE'], Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_NRLOGINTCENT',Param::PARAM_OUTPUT, null,                 Param::PARAM_TYPE_STR,20));
        
        			$result = $procedure->execute();
        			if(isset($result['P_NRLOGINTCENT'])){
        			    array_push($nrLog, $result['P_NRLOGINTCENT']);
        			}
                    $this->connection->commit();
                }catch(\Exception $e){
                    $this->connection->rollback();
                    throw $e;
                }
            }
            $this->apagaListaPratos($lista);
            $erros = $this->buscaLog($nrLog);
            $this->apagaLog($nrLog);
            $response->addDataSet(new DataSet("ERRO_PROC", $erros));
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function criaListaPratos($pratos){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        try{
            $this->connection->beginTransaction();
            foreach($pratos as $prato){
                $sql = "INSERT INTO LISTA_PRATO (NRLISTA, CDPRATO, CDFILAUXPRAT) VALUES (:NRLISTA, :CDPRATO, :CDFILAUXPRAT)";
                $params = array(
                    "NRLISTA"      => $nrlista,
                    "CDPRATO"      => $prato["CDPRATO"],
                    "CDFILAUXPRAT" => $prato["CDFILAUXPRAT"]
                );
                $query = $this->connection->prepare($sql);
                $query->execute($params);
            }
            $this->entityManager->commit();
            return $nrlista;
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function apagaListaPratos($nrlista) {
        try{
            $this->connection->beginTransaction();
            $params = array(
                "NRLISTA" => $nrlista
            );
            
            $sql = "DELETE FROM LISTA_PRATO WHERE NRLISTA =:NRLISTA";
            $query = $this->connection->prepare($sql);
            $query->execute($params);
            
            $this->entityManager->commit();
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function buscaLog($logs){
        $dados = array();
        if(sizeof($logs)> 0){
            foreach($logs as $nrLog){
                $conditions = [
                    array(
                        "columnName" => "P_NRLOGINTCENT",
                        "operator"   => '=',
                        "value"      => $nrLog
                    )
                ];
                $dado = $this->crudService->getFilteredDataSet('pla06000_proc_log', $conditions);
                array_push($dados, $dado->getRows());
            }
        }
        return $dados;
    }
    
    public function apagaLog($logs){
        if(sizeof($logs) > 0){
            try{
                $this->connection->beginTransaction();
                $params = array(
                    "NRLOGINTCENT" => join('_', $logs)
                );
                
                $sql = "DELETE FROM LOGINTRECCENT WHERE INSTR(:NRLOGINTCENT, NRLOGINTCENT) <> 0";
                $query = $this->connection->prepare($sql);
                $query->execute($params);
                
                $this->entityManager->commit();
            } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        }
    }
}